__author__ = "Jianfeng Sun"
__version__ = "v1.0"
__copyright__ = "Copyright 2022"
__license__ = "MIT"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import numpy as np
import pandas as pd
from rdkit import Chem
from deepsmirud.Utils import fasta
from deepsmirud import Biochar


def fetch(
        smile_fpn,
        fasta_fpn,
):
    smile = pd.read_csv(
        smile_fpn,
        sep='\t',
        header=None,
    )
    print(smile)
    mol = Chem.MolFromSmiles(smile.loc[0, 0])
    print(mol)
    v = [[] for _ in range(1)]

    ntseq = fasta(fasta_fpn=fasta_fpn)

    ns_ = Biochar.ns(ntseq)
    ns_ = [*ns_.values()]
    ns_ = [np.float32(i) for i in ns_]
    for j in range(4):
        v[0].append(ns_[j])

    ds_ = Biochar.ds(ntseq)
    ds_ = [*ds_.values()]
    for j in range(16):
        v[0].append(np.float32(ds_[j]))

    ts_ = Biochar.ts(ntseq)
    ts_ = [*ts_.values()]
    for j in range(64):
        v[0].append(np.float32(ts_[j]))

    qs_ = Biochar.qs(ntseq)
    qs_ = [*qs_.values()]
    for j in range(256):
        v[0].append(np.float32(qs_[j]))

    bseqs_ = Biochar.bseqs(ntseq, k=3)
    bseqs_ = [*bseqs_.values()]
    for j in range(16):
        v[0].append(np.float32(bseqs_[j]))

    aseqs_ = Biochar.aseqs(ntseq)
    aseqs_ = [*aseqs_.values()]
    for j in range(4):
        v[0].append(np.float32(aseqs_[j]))

    pc10 = Biochar.bcs(mol)

    for _, t in enumerate(pc10):
        v[0].append(np.float32(t))

    pc2 = Biochar.crippen(mol)
    for _, t in enumerate(pc2):
        v[0].append(np.float32(t))

    fp_morgan = Biochar.fp(mol)
    for _, e in enumerate(fp_morgan):
        v[0].append(np.int(e))
    v = np.array(v)
    return v[:, 0: 1369].astype(np.float32)


__author__ = "Jianfeng Sun"
__version__ = "v1.0"
__copyright__ = "Copyright 2022"
__license__ = "MIT"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import numpy as np
import pandas as pd
import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from deepsmirud.Pipe import fetch


def drestruct(data, met):
    if met == 'ConvMixer64':
        return np.reshape(data, [-1, 37, 37, 1])
    else:
        return data


class predict(object):

    def __init__(
            self,
            smile_fpn,
            fasta_fpn,
            method,
            model_fp,
            sv_fpn,
    ):
        self.method = method
        self.model_fp = model_fp
        self.sv_fpn = sv_fpn
        self.ind = fetch(
            smile_fpn=smile_fpn,
            fasta_fpn=fasta_fpn,
        )

    def m2(self, ):
        loaded_model = tf.keras.models.load_model(self.model_fp)
        pret = loaded_model.predict(drestruct(self.ind, self.method), batch_size=1)
        t = []
        if pret[0][1] > pret[0][0]:
            t.append([pret[0][1], 'Upregulation'])
        else:
            t.append([pret[0][0], 'Downregulation'])
        df = pd.DataFrame(t, columns=['prob', 'predicted_type'])
        df.to_csv(
            self.sv_fpn,
            sep='\t',
            header=True,
            index=False,
        )
        return df

    def m1(self, ):
        sess = tf.Session()
        saver = tf.train.import_meta_graph(self.model_fp + '.meta')
        saver.restore(sess, self.model_fp)
        tf_graph = tf.get_default_graph()
        x = tf_graph.get_tensor_by_name("x:0")
        Placeholder = tf_graph.get_tensor_by_name("Placeholder:0")
        prediction = tf_graph.get_tensor_by_name("pred_softmax:0")
        pret = self.m1w(
            sess=sess,
            x=x,
            Placeholder=Placeholder,
            prediction=prediction,
            x_test=self.ind,
        )
        t = []
        if pret[0][1] > pret[0][0]:
            t.append([pret[0][1], 'Upregulation'])
        else:
            t.append([pret[0][0], 'Downregulation'])
        df = pd.DataFrame(t, columns=['prob', 'predicted_type'])
        df.to_csv(
            self.sv_fpn,
            sep='\t',
            header=True,
            index=False,
        )
        return df

    def m1w(self, sess, x, prediction, x_test, Placeholder):
        pds = []
        indict = {x: x_test, Placeholder: 1}
        p_tmp = sess.run(prediction, feed_dict=indict)
        pds.append(p_tmp)
        return pds[0]


if __name__ == "__main__":
    p = predict(
        # 'data/example/5743.txt',
        # 'data/example/MIMAT0000539.fasta',

        # 'data/example/84093.txt',
        # 'data/example/MIMAT0009203.fasta',
        
        # 'data/example/148124.txt',
        # 'data/example/MIMAT0000098.fasta',

        smile_fpn='data/example/5757.txt',
        fasta_fpn='data/example/MIMAT0000066.fasta',

        method='AlexNet',
        # method='BiRNN',
        # method='RNN',
        # method='Seq2Seq',
        # method='CNN',
        # method='ConvMixer64',
        # method='DSConv',
        # method='LSTMCNN',
        # method='MobileNet',
        # method='ResNet18',
        # method='ResNet50',
        # method='SEResNet',

        model_fp='model/alexnet/alexnet',
        # model_fp='model/birnn/birnn',
        # model_fp='model/cnn',
        # model_fp='model/convmixer64',
        # model_fp='model/dsconv',
        # model_fp='model/lstmcnn',
        # model_fp='model/mobilenet',
        # model_fp='model/resnet_prea18',
        # model_fp='model/resnet_prea50',
        # model_fp='model/rnn/rnn',
        # model_fp='model/seq2seq/seq2seq',
        # model_fp='model/seresnet',

        sv_fpn='./ass',
    )
    # print(p.m2())
    print(p.m1())



from setuptools import setup, find_packages

setup(
    name="deepsmirud",
    # version="0.0.1",
    version="0.0.0.0.21",
    keywords=("pip", "deepsmirud"),
    description="deepsmirud",
    long_description="deep learning SM-miR relation",
    license="MIT",

    url="https://github.com/2003100127",
    author="Jianfeng Sun",
    author_email="jianfeng.sunmt@gmail.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    python_requires='>3.6',
    install_requires=[
        # 'pandas==1.3.5',
        # 'numpy==1.19.5',
        # 'biopython==1.79',
        # 'pyfiglet==0.8.post1',
        # 'click==8.1.3',
        # 'tensorflow==2.6.0',
        # 'rdkit-pypi==2022.3.1',

        'pandas',
        'numpy',
        'biopython',
        'pyfiglet',
        'click',
        'rdkit-pypi',
        'tensorflow',
    ],
    entry_points={
        'console_scripts': [
            'deepsmirud=deepsmirud.Main:main',
            'deepsmirud_download=deepsmirud.Main:download',
        ],
    }
)